"""Shared Rich markup lines for CLI TUIs (Textual Static / RichLog)."""

# Top brand bar (run TUI + model picker)
BRAND_MARKUP_LINE = (
    "[bold #58a6ff]REMOROO[/]  [dim]autonomous engineer[/]  [dim]·[/]  "
    "[dim italic]Ship faster with agents[/]  [dim]·[/]  [#8b949e]remoroo.com[/]"
)

BRAND_MARKUP_MODEL_PICKER = (
    "[bold #58a6ff]REMOROO[/]  [dim]model[/]  [dim]·[/]  "
    "[dim italic]Ship faster with agents[/]  [dim]·[/]  [#8b949e]remoroo.com[/]"
)
